#!/usr/bin/python
# -*- coding:utf-8 -*-

#  Copyright © 2011  B. Clausius <barcc@gmx.de>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License,  or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, os

import gobject
import gtk
import glib
import appindicator

from call import call, popen


class Countdown:
    factors = [0.5, 1, 2]
    
    def __init__(self):
        self.time = 0
        self.timer_handler = None
        self.increment = [10*60, 20*60]
        self.direction = True
        self.position = len(self.factors) - 1
        
    reset = __init__
    
    def up(self):
        self.time += self.increment[1]
        inc1 = abs(self.increment[1])
        inc1new0 = int(inc1 * self.factors[0]) or 1
        if self.direction:
            if self.position < len(self.factors) - 1:
                self.position += 1
            self.increment[0] = -inc1new0
            self.increment[1] = int(inc1 * self.factors[self.position])
        else:
            self.direction = True
            self.position = 0
            self.increment[0] = -inc1new0
            self.increment[1] = inc1new0
        
    def down(self):
        self.time += self.increment[0]
        inc0 = abs(self.increment[0])
        inc0new0 = int(inc0 * self.factors[0]) or 1
        if self.direction:
            self.direction = False
            self.position = 0
            self.increment[0] = -inc0new0
            self.increment[1] = inc0new0
        else:
            if self.position < len(self.factors) - 1:
                self.position += 1
            self.increment[0] = -int(inc0 * self.factors[self.position])
            self.increment[1] = inc0new0
            
    def _label(self, inc):
        sec = abs(inc) % 60
        min = abs(inc) // 60
        sign = '-' if inc < 0 else '+'
        if sec:
            return '{}{}:{:02}'.format(sign, min, sec)
        else:
            return '{}{} min'.format(sign, min)
    def labels(self):
        return (self._label(self.increment[0]),
                self._label(self.increment[1]))

def show_time(indicator, countdown):
    if not countdown.timer_handler:
        indicator.set_label('')
    elif countdown.time >= 0:
        indicator.set_label('{0}:{1:02}'.format(countdown.time // 60, countdown.time % 60))
    else:
        indicator.set_label('-{0}:{1:02}'.format((-countdown.time // 60), (-countdown.time % 60)))

def on_menuitem_increment(unused_menuitem, indicator, menuitems, countdown, direction):
    if direction:
        countdown.up()
    else:
        countdown.down()
    if countdown.time >= 0:
        indicator.set_status(appindicator.STATUS_ACTIVE)
    if not countdown.timer_handler:
        countdown.timer_handler = glib.timeout_add(1000, on_timer, indicator, countdown)
    menuitems[0].props.label, menuitems[1].props.label = countdown.labels()
    show_time(indicator, countdown)

def on_menuitem_toggle_pause(menuitem, indicator, countdown):
    if countdown.timer_handler:
        glib.source_remove(countdown.timer_handler)
        countdown.timer_handler = None
        menuitem.set_label('Continue')
    elif countdown.time > 0:
        countdown.timer_handler = glib.timeout_add(1000, on_timer, indicator, countdown)
        menuitem.set_label('Pause')

def on_menuitem_stop(unused_menuitem, indicator, menuitems, countdown):
    if countdown.timer_handler:
        glib.source_remove(countdown.timer_handler)
        countdown.timer_handler = None
    countdown.reset()
    indicator.set_status(appindicator.STATUS_ACTIVE)
    menuitems[0].props.label, menuitems[1].props.label = countdown.labels()
    show_time(indicator, countdown)

def on_timer(indicator, countdown):
    countdown.time -= 1
    if countdown.time == 0:
        indicator.set_status(appindicator.STATUS_ATTENTION)
        popen('messagesound')
    show_time(indicator, countdown)
    return True

def main():
    countdown = Countdown()

    indicator = appindicator.Indicator("appmenu-timer", "indicator-messages",
                                  appindicator.CATEGORY_APPLICATION_STATUS)
    indicator.set_status(appindicator.STATUS_ACTIVE)
    indicator.set_icon("apple-green")
    indicator.set_attention_icon("apple-red")

    # create a menu
    menu = gtk.Menu()
    labels = countdown.labels()
    menuitems = gtk.MenuItem(labels[0]), gtk.MenuItem(labels[1])
    menu.append(menuitems[0])
    menu.append(menuitems[1])

    menuitems[0].connect("activate", on_menuitem_increment, indicator, menuitems, countdown, False)
    menuitems[0].show()
    menuitems[1].connect("activate", on_menuitem_increment, indicator, menuitems, countdown, True)
    menuitems[1].show()

    menu_item = gtk.MenuItem('Stop')
    menu.append(menu_item)
    menu_item.connect("activate", on_menuitem_stop, indicator, menuitems, countdown)
    menu_item.show()

    menu_item = gtk.MenuItem('Beenden')
    menu.append(menu_item)
    menu_item.connect("activate", gtk.main_quit, countdown)
    menu_item.show()

    indicator.set_menu(menu)

    gtk.main()
    
    
if __name__ == "__main__":
    main()

